from fastapi import APIRouter, Request, Response

from openaoe.backend.model.google import GooglePalmChatBody
from openaoe.backend.service.service_google import palm_chat_svc, Gemma
from openaoe.backend.model.openaoe import AoeChatBody

router = APIRouter()


@router.post("/v1/palm/chat", tags=["PaLM"])
async def palm_chat(body: GooglePalmChatBody):
    """
    chat api for google PaLM model
    @param body: request body
    @return: response
    """
    ret = await palm_chat_svc(body)
    return ret


@router.post("/v1/gemma/chat", tags=["Gemma"])
async def gemma_chat(body: AoeChatBody, request: Request, response: Response):
    return await Gemma(request, response).chat(body)



from typing import Optional, List, Literal, Dict
from pydantic import BaseModel, Field


class Context(BaseModel):
    send_type: str = 'assistant'
    sender_type: str = "assistant"
    text: str = ''


class RoleMeta(BaseModel):
    user_name: Optional[str] = 'user'
    bot_name: Optional[str] = 'assistant'


class AoeChatBody(BaseModel):
    """
    OpenAOE general request body
    """
    model: str
    prompt: str
    messages: Optional[List[Context]] = []
    role_meta: Optional[RoleMeta] = None
    type: Optional[Literal['text', 'json']] = 'json'
    stream: Optional[bool] = True


"""
ref. to https://github.com/ollama/ollama/blob/main/docs/api.md
Parameters
model: (required) the model name
messages: the messages of the chat, this can be used to keep a chat memory
The message object has the following fields:

role: the role of the message, either system, user or assistant
content: the content of the message
images (optional): a list of images to include in the message (for multimodal models such as llava)
Advanced parameters (optional):

format: the format to return a response in. Currently the only accepted value is json
options: additional model parameters listed in the documentation for the Modelfile such as temperature
template: the prompt template to use (overrides what is defined in the Modelfile)
stream: if false the response will be returned as a single response object, rather than a stream of objects
keep_alive: controls how long the model will stay loaded into memory following the request (default: 5m)
"""


class OllamaMessage(BaseModel):
    role: Literal["user", "system", "assistant"] = "user"
    content: str
    images: Optional[List[str]] = None  # img in base64


class OllamaChatBody(BaseModel):
    model: str
    messages: List[OllamaMessage] = Field(default_factory=List)
    options: Optional[Dict] = Field(default_factory=dict)
    template: Optional[str] = None
    stream: Optional[bool] = True
    keep_alive: Optional[str] = '5m'




"""
ref. to https://github.com/ollama/ollama/blob/main/docs/api.md
Parameters
model: (required) the model name
messages: the messages of the chat, this can be used to keep a chat memory
The message object has the following fields:

role: the role of the message, either system, user or assistant
content: the content of the message
images (optional): a list of images to include in the message (for multimodal models such as llava)
Advanced parameters (optional):

format: the format to return a response in. Currently the only accepted value is json
options: additional model parameters listed in the documentation for the Modelfile such as temperature
template: the prompt template to use (overrides what is defined in the Modelfile)
stream: if false the response will be returned as a single response object, rather than a stream of objects
keep_alive: controls how long the model will stay loaded into memory following the request (default: 5m)
"""

from openaoe.backend.model.openaoe import OllamaChatBody, OllamaMessage


class MistralChatBody(OllamaChatBody):
    pass



import json

import requests
from fastapi import Request, Response
from sse_starlette import EventSourceResponse

from openaoe.backend.config.biz_config import get_base_url, get_api_path
from openaoe.backend.config.constant import PROVIDER_MISTRAL
from openaoe.backend.model.openaoe import AoeChatBody, OllamaMessage
from openaoe.backend.model.mistral import MistralChatBody
from openaoe.backend.util.convert import body_convert

from openaoe.backend.util.log import log
logger = log(__name__)


class Mistral:
    def __init__(self, request: Request, response: Response):
        self.request = request
        self.response = response

    async def chat(self, body: AoeChatBody):
        chat_body = body_convert(body)
        chat_url = get_base_url(PROVIDER_MISTRAL, body.model) + get_api_path(PROVIDER_MISTRAL, body.model)
        return self.chat_response_streaming(chat_url, chat_body)

    def chat_response_streaming(self, chat_url: str, chat_body: MistralChatBody):
        async def do_response_streaming():
            try:
                res = requests.post(chat_url, json=json.loads(chat_body.model_dump_json()), stream=True)
                if res:
                    for chunk in res.iter_content(chunk_size=512, decode_unicode=True):
                        chunk = bytes.decode(chunk)
                        logger.info(f"chunk: {chunk}")
                        chunk_json = json.loads(chunk)
                        yield json.dumps({
                            "success": True,
                            "msg": chunk_json.get("message").get("content")
                        }, ensure_ascii=False)
            except Exception as e:
                logger.error(f"{e}")
                yield json.dumps(
                    {
                        "success": "false",
                        "msg": f"from backend: {e}"
                    }
                )

        return EventSourceResponse(do_response_streaming())







from fastapi import APIRouter, Request, Response

from openaoe.backend.service.mistral import Mistral
from openaoe.backend.model.openaoe import AoeChatBody

router = APIRouter()


@router.post("/v1/mistral/chat", tags=["Mistral"])
async def mistral_chat(body: AoeChatBody, request: Request, response: Response):
    """
    chat api for Mistral 7b model
    :param body: request body
    :param request: request
    :param response: response
    :return
    """
    return await Mistral(request, response).chat(body)



from openaoe.backend.model.google import GemmaChatBody
from openaoe.backend.model.openaoe import AoeChatBody, OllamaMessage, OllamaChatBody


def body_convert(body: AoeChatBody):
    target_body = None
    msgs = []
    if "gemma" in body.model:
        for msg in body.messages:
            m = OllamaMessage(role=msg.sender_type if msg.sender_type != 'bot' else "assistant", content=msg.text)
            msgs.append(m)
        last_m = OllamaMessage(role='user', content=body.prompt)
        msgs.append(last_m)
        target_body = GemmaChatBody(
            model="gemma:7b",
            messages=msgs
        )
    elif "mistral" in body.model:
        for msg in body.messages:
            m = OllamaMessage(role=msg.sender_type if msg.sender_type != 'bot' else "assistant", content=msg.text)
            msgs.append(m)
        last_m = OllamaMessage(role='user', content=body.prompt)
        msgs.append(last_m)
        target_body = OllamaChatBody(
            model="mistral",
            messages=msgs
        )
    return target_body



import os.path
import re
import sys
from os.path import exists


def get_file_content(file_name):
    """
    get the file content by file_name
    """
    if not os.path.isfile(file_name):
        print(f"ERROR: invalid file: {file_name}, not exist or not file")
        sys.exit(-1)

    with open(file_name, encoding='utf-8') as f:
        content = f.read()
    return content


def parse_requirements(fname='requirements.txt', with_version=True):
    """
    Parse the package dependencies listed in a requirements file but strips
    specific versioning information.
    @param fname: path to requirements file
    @param with_version: if True include version specs
    @return: List[str]: list of requirements items
    """

    require_fpath = fname

    def parse_line(line):
        """Parse information from a line in a requirements text file."""
        if line.startswith('-r '):
            # Allow specifying requirements in other files
            target = line.split(' ')[1]
            for info in parse_require_file(target):
                yield info
        else:
            info = {'line': line}
            if line.startswith('-e '):
                info['package'] = line.split('#egg=')[1]
            elif '@git+' in line:
                info['package'] = line
            else:
                # Remove versioning from the package
                pat = '(' + '|'.join(['>=', '==', '>']) + ')'
                parts = re.split(pat, line, maxsplit=1)
                parts = [p.strip() for p in parts]

                info['package'] = parts[0]
                if len(parts) > 1:
                    op, rest = parts[1:]
                    if ';' in rest:
                        # Handle platform specific dependencies
                        # http://setuptools.readthedocs.io/en/latest/setuptools.html#declaring-platform-specific-dependencies
                        version, platform_deps = map(str.strip,
                                                     rest.split(';'))
                        info['platform_deps'] = platform_deps
                    else:
                        version = rest  # NOQA
                    info['version'] = (op, version)
            yield info

    def parse_require_file(fpath):
        with open(fpath, 'r') as f:
            for line in f.readlines():
                line = line.strip()
                if line and not line.startswith('#'):
                    for info in parse_line(line):
                        yield info

    def gen_packages_items():
        if exists(require_fpath):
            startswith = sys.version.startswith('3.9')
            for info in parse_require_file(require_fpath):
                parts = [info['package']]
                if with_version and 'version' in info:
                    parts.extend(info['version'])
                if not startswith:
                    # apparently package_deps are broken in 3.4
                    platform_deps = info.get('platform_deps')
                    if platform_deps is not None:
                        parts.append(';' + platform_deps)
                item = ''.join(parts)
                yield item

    packages = list(gen_packages_items())
    return packages
